@@ -34,23 +34,26 @@ tf <- function() {
     silent = TRUE)
   }, add = TRUE)
 
-  # check server
-  testUrl <- "https://www.clinicaltrialsregister.eu/ctr-search/search"
-  testGet <- function() try(httr::HEAD(testUrl, httr::timeout(10L)), silent = TRUE)
-  testOnce <- testGet()
-
+  # check certificates
   if (inherits(testOnce, "try-error") &&
       grepl("SSL certificate.*local issuer certificate", testOnce)) {
-    message("Switching off certificate verification")
+    # message("Switching off certificate verification")
     httr::set_config(httr::config(ssl_verifypeer = FALSE))
     testOnce <- testGet()
   }
-  if (inherits(testOnce, "try-error") ||
-      httr::status_code(testOnce) != 200L
-  ) return(exit_file("Reason: EUCTR not working"))
 
   # do tests
   source("ctrdata_euctr.R", local = TRUE)
 
 }
+
+# check server
+testUrl <- "https://www.clinicaltrialsregister.eu/ctr-search/search"
+testGet <- function() try(httr::HEAD(testUrl, httr::timeout(10L)), silent = TRUE)
+testOnce <- testGet()
+if (inherits(testOnce, "try-error") ||
+    httr::status_code(testOnce) != 200L
+) return(exit_file("Reason: EUCTR not working"))
+
+# test
 tf()
